@@ -1,9 +1,9 @@
 /*
- * "$Id: classes.c 8859 2009-11-09 23:01:17Z mike $"
+ * "$Id: classes.c 9470 2011-01-11 07:05:58Z mike $"
  *
- *   Class status CGI for the Common UNIX Printing System (CUPS).
+ *   Class status CGI for CUPS.
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -147,7 +147,21 @@ main(int  argc,				/* I - Number of command-line arguments */
   }
   else if (pclass)
   {
-    if (!strcmp(op, "start-class"))
+    if (!*op)
+    {
+      const char *server_port = getenv("SERVER_PORT");
+					/* Port number string */
+      int	port = atoi(server_port ? server_port : "0");
+      					/* Port number */
+      char	uri[1024];		/* URL */
+
+      httpAssembleURIf(HTTP_URI_CODING_ALL, uri, sizeof(uri),
+		       getenv("HTTPS") ? "https" : "http", NULL,
+		       getenv("SERVER_NAME"), port, "/classes/%s", pclass);
+
+      printf("Location: %s\n\n", uri);
+    }
+    else if (!strcmp(op, "start-class"))
       do_class_op(http, pclass, IPP_RESUME_PRINTER, cgiText(_("Resume Class")));
     else if (!strcmp(op, "stop-class"))
       do_class_op(http, pclass, IPP_PAUSE_PRINTER, cgiText(_("Pause Class")));
@@ -540,5 +554,5 @@ show_class(http_t     *http,		/* I - Connection to server */
 
 
 /*
- * End of "$Id: classes.c 8859 2009-11-09 23:01:17Z mike $".
+ * End of "$Id: classes.c 9470 2011-01-11 07:05:58Z mike $".
  */
