@@ -1,9 +1,9 @@
 /*
- * "$Id: http-private.h 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: http-private.h 9503 2011-01-22 00:07:22Z mike $"
  *
- *   Private HTTP definitions for the Common UNIX Printing System (CUPS).
+ *   Private HTTP definitions for CUPS.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -98,6 +98,7 @@ extern BIO_METHOD *_httpBIOMethods(void);
  * The GNU TLS library is more of a "bare metal" SSL/TLS library...
  */
 #    include <gnutls/gnutls.h>
+#    include <gcrypt.h>
 
 typedef struct
 {
@@ -258,13 +259,20 @@ extern void	_cups_freeifaddrs(struct ifaddrs *addrs);
 #  endif /* !WIN32 */
 
 /*
- * Common URI encoding function...
+ * Prototypes...
  */
 
-extern char	*_httpEncodeURI(char *dst, const char *src, size_t dstsize);
-
+extern int		_httpAddrPort(http_addr_t *addr);
+extern http_t		*_httpCreate(const char *host, int port,
+			             http_encryption_t encryption);
+extern char		*_httpEncodeURI(char *dst, const char *src,
+			                size_t dstsize);
+extern const char	*_httpResolveURI(const char *uri, char *resolved_uri,
+			                 size_t resolved_size, int log);
+extern int		_httpUpdate(http_t *http, http_status_t *status);
+extern int		_httpWait(http_t *http, int msec, int usessl);
 #endif /* !_CUPS_HTTP_PRIVATE_H_ */
 
 /*
- * End of "$Id: http-private.h 7721 2008-07-11 22:48:49Z mike $".
+ * End of "$Id: http-private.h 9503 2011-01-22 00:07:22Z mike $".
  */
