@@ -1,9 +1,9 @@
 /*
- * "$Id: mark.c 8912 2009-12-08 02:13:42Z mike $"
+ * "$Id: mark.c 9426 2011-01-06 22:48:42Z mike $"
  *
- *   Option marking routines for the Common UNIX Printing System (CUPS).
+ *   Option marking routines for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -29,13 +29,11 @@
  *   ppdFirstOption()      - Return the first option in the PPD file.
  *   ppdNextOption()       - Return the next option in the PPD file.
  *   _ppdParseOptions()    - Parse options from a PPD file.
- *   debug_marked()        - Output the marked array to stdout...
+ *   ppd_debug_marked()    - Output the marked array to stdout...
  *   ppd_defaults()        - Set the defaults for this group and all sub-groups.
  *   ppd_mark_choices()    - Mark one or more option choices from a string.
  *   ppd_mark_option()     - Quickly mark an option without checking for
  *                           conflicts.
- *   ppd_mark_size()       - Quickly mark a page size without checking for
- *                           conflicts.
  */
 
 /*
@@ -45,7 +43,8 @@
 #include "cups.h"
 #include "string.h"
 #include "debug.h"
-#include "pwgmedia.h"
+#include "ppd-private.h"
+#include "pwg-private.h"
 
 
 /*
@@ -53,15 +52,14 @@
  */
 
 #ifdef DEBUG
-static void	debug_marked(ppd_file_t *ppd, const char *title);
+static void	ppd_debug_marked(ppd_file_t *ppd, const char *title);
 #else
-#  define	debug_marked(ppd,title)
+#  define	ppd_debug_marked(ppd,title)
 #endif /* DEBUG */
 static void	ppd_defaults(ppd_file_t *ppd, ppd_group_t *g);
 static void	ppd_mark_choices(ppd_file_t *ppd, const char *s);
 static void	ppd_mark_option(ppd_file_t *ppd, const char *option,
 		                const char *choice);
-static void	ppd_mark_size(ppd_file_t *ppd, const char *size);
 
 
 /*
@@ -78,41 +76,20 @@ cupsMarkOptions(
     int           num_options,		/* I - Number of options */
     cups_option_t *options)		/* I - Options */
 {
-  int		i, j, k;		/* Looping vars */
+  int		i, j;			/* Looping vars */
   char		*ptr,			/* Pointer into string */
 		s[255];			/* Temporary string */
   const char	*val,			/* Pointer into value */
 		*media,			/* media option */
-		*media_col,		/* media-col option */
-		*page_size;		/* PageSize option */
+		*output_bin,		/* output-bin option */
+		*output_mode,		/* output-mode option */
+		*page_size,		/* PageSize option */
+		*ppd_keyword,		/* PPD keyword */
+		*print_quality,		/* print-quality option */
+		*sides;			/* sides option */
   cups_option_t	*optptr;		/* Current option */
-  ppd_option_t	*option;		/* PPD option */
   ppd_attr_t	*attr;			/* PPD attribute */
-  static const char * const duplex_options[] =
-		{			/* Duplex option names */
-		  "Duplex",		/* Adobe */
-		  "EFDuplex",		/* EFI */
-		  "EFDuplexing",	/* EFI */
-		  "KD03Duplex",		/* Kodak */
-		  "JCLDuplex"		/* Samsung */
-		};
-  static const char * const duplex_one[] =
-		{			/* one-sided names */
-		  "None",
-		  "False"
-		};
-  static const char * const duplex_two_long[] =
-		{			/* two-sided-long-edge names */
-		  "DuplexNoTumble",	/* Adobe */
-		  "LongEdge",		/* EFI */
-		  "Top"			/* EFI */
-		};
-  static const char * const duplex_two_short[] =
-		{			/* two-sided-long-edge names */
-		  "DuplexTumble",	/* Adobe */
-		  "ShortEdge",		/* EFI */
-		  "Bottom"		/* EFI */
-		};
+  _pwg_t	*pwg;			/* PWG mapping data */
 
 
  /*
@@ -122,79 +99,42 @@ cupsMarkOptions(
   if (!ppd || num_options <= 0 || !options)
     return (0);
 
-  debug_marked(ppd, "Before...");
+  ppd_debug_marked(ppd, "Before...");
 
  /*
-  * Do special handling for media, media-col, and PageSize...
+  * Do special handling for finishings, media, output-bin, output-mode,
+  * print-color-mode, print-quality, and PageSize...
   */
 
-  media     = cupsGetOption("media", num_options, options);
-  media_col = cupsGetOption("media-col", num_options, options);
-  page_size = cupsGetOption("PageSize", num_options, options);
+  media         = cupsGetOption("media", num_options, options);
+  output_bin    = cupsGetOption("output-bin", num_options, options);
+  output_mode   = cupsGetOption("output-mode", num_options, options);
+  page_size     = cupsGetOption("PageSize", num_options, options);
+  print_quality = cupsGetOption("print-quality", num_options, options);
+  sides         = cupsGetOption("sides", num_options, options);
 
-  if (media_col && (!page_size || !page_size[0]))
+  if ((media || output_bin || output_mode || print_quality || sides) &&
+      !ppd->pwg)
   {
    /*
-    * Pull out the corresponding media size from the media-col value...
+    * Load PWG mapping data as needed...
     */
 
-    int			num_media_cols,	/* Number of media-col values */
-    			num_media_sizes;/* Number of media-size values */
-    cups_option_t	*media_cols,	/* media-col values */
-			*media_sizes;	/* media-size values */
-
-
-    num_media_cols = cupsParseOptions(media_col, 0, &media_cols);
-
-    if ((val = cupsGetOption("media-key", num_media_cols, media_cols)) != NULL)
-      media = val;
-    else if ((val = cupsGetOption("media-size", num_media_cols,
-                                  media_cols)) != NULL)
-    {
-     /*
-      * Lookup by dimensions...
-      */
-
-      double		width,		/* Width in points */
-			length;		/* Length in points */
-      struct lconv	*loc;		/* Locale data */
-      _cups_pwg_media_t	*pwgmedia;	/* PWG media name */
-
-
-      num_media_sizes = cupsParseOptions(val, 0, &media_sizes);
-      loc             = localeconv();
-
-      if ((val = cupsGetOption("x-dimension", num_media_sizes,
-                               media_sizes)) != NULL)
-        width = _cupsStrScand(val, NULL, loc) * 2540.0 / 72.0;
-      else
-        width = 0.0;
-
-      if ((val = cupsGetOption("y-dimension", num_media_sizes,
-                               media_sizes)) != NULL)
-        length = _cupsStrScand(val, NULL, loc) * 2540.0 / 72.0;
-      else
-        length = 0.0;
-
-      if ((pwgmedia = _cupsPWGMediaBySize(width, length)) != NULL)
-        media = pwgmedia->pwg;
-
-      cupsFreeOptions(num_media_sizes, media_sizes);
-    }
-
-    cupsFreeOptions(num_media_cols, media_cols);
+    ppd->pwg = _pwgCreateWithPPD(ppd);
   }
 
+  pwg = (_pwg_t *)ppd->pwg;
+
   if (media)
   {
    /*
-    * Loop through the option string, separating it at commas and
-    * marking each individual option as long as the corresponding
-    * PPD option (PageSize, InputSlot, etc.) is not also set.
+    * Loop through the option string, separating it at commas and marking each
+    * individual option as long as the corresponding PPD option (PageSize,
+    * InputSlot, etc.) is not also set.
     *
-    * For PageSize, we also check for an empty option value since
-    * some versions of MacOS X use it to specify auto-selection
-    * of the media based solely on the size.
+    * For PageSize, we also check for an empty option value since some versions
+    * of MacOS X use it to specify auto-selection of the media based solely on
+    * the size.
     */
 
     for (val = media; *val;)
@@ -215,23 +155,114 @@ cupsMarkOptions(
       */
 
       if (!page_size || !page_size[0])
-	ppd_mark_size(ppd, s);
+      {
+        if (!strncasecmp(s, "Custom.", 7) || ppdPageSize(ppd, s))
+          ppd_mark_option(ppd, "PageSize", s);
+        else if ((ppd_keyword = _pwgGetPageSize(pwg, NULL, s, NULL)) != NULL)
+	  ppd_mark_option(ppd, "PageSize", ppd_keyword);
+      }
 
-      if (cupsGetOption("InputSlot", num_options, options) == NULL)
-	ppd_mark_option(ppd, "InputSlot", s);
+      if (pwg && pwg->source_option &&
+          !cupsGetOption(pwg->source_option, num_options, options) &&
+	  (ppd_keyword = _pwgGetInputSlot(pwg, NULL, s)) != NULL)
+	ppd_mark_option(ppd, pwg->source_option, ppd_keyword);
 
-      if (cupsGetOption("MediaType", num_options, options) == NULL)
-	ppd_mark_option(ppd, "MediaType", s);
+      if (!cupsGetOption("MediaType", num_options, options) &&
+	  (ppd_keyword = _pwgGetMediaType(pwg, NULL, s)) != NULL)
+	ppd_mark_option(ppd, "MediaType", ppd_keyword);
+    }
+  }
 
-      if (cupsGetOption("EFMediaType", num_options, options) == NULL)
-	ppd_mark_option(ppd, "EFMediaType", s);		/* EFI */
+  if (pwg)
+  {
+    if (!cupsGetOption("com.apple.print.DocumentTicket.PMSpoolFormat",
+                       num_options, options) &&
+        !cupsGetOption("APPrinterPreset", num_options, options) &&
+        (output_mode || print_quality))
+    {
+     /*
+      * Map output-mode and print-quality to a preset...
+      */
 
-      if (cupsGetOption("EFMediaQualityMode", num_options, options) == NULL)
-	ppd_mark_option(ppd, "EFMediaQualityMode", s);	/* EFI */
+      _pwg_output_mode_t	pwg_om;	/* output-mode index */
+      _pwg_print_quality_t	pwg_pq;	/* print-quality index */
+      cups_option_t		*preset;/* Current preset option */
 
-      if (!strcasecmp(s, "manual") &&
-	  !cupsGetOption("ManualFeed", num_options, options))
-	ppd_mark_option(ppd, "ManualFeed", "True");
+      if (output_mode && !strcmp(output_mode, "monochrome"))
+	pwg_om = _PWG_OUTPUT_MODE_MONOCHROME;
+      else
+	pwg_om = _PWG_OUTPUT_MODE_COLOR;
+
+      if (print_quality)
+      {
+	pwg_pq = atoi(print_quality) - IPP_QUALITY_DRAFT;
+	if (pwg_pq < _PWG_PRINT_QUALITY_DRAFT)
+	  pwg_pq = _PWG_PRINT_QUALITY_DRAFT;
+	else if (pwg_pq > _PWG_PRINT_QUALITY_HIGH)
+	  pwg_pq = _PWG_PRINT_QUALITY_HIGH;
+      }
+      else
+	pwg_pq = _PWG_PRINT_QUALITY_NORMAL;
+
+      if (pwg->num_presets[pwg_om][pwg_pq] == 0)
+      {
+       /*
+	* Try to find a preset that works so that we maximize the chances of us
+	* getting a good print using IPP attributes.
+	*/
+
+	if (pwg->num_presets[pwg_om][_PWG_PRINT_QUALITY_NORMAL] > 0)
+	  pwg_pq = _PWG_PRINT_QUALITY_NORMAL;
+	else if (pwg->num_presets[_PWG_OUTPUT_MODE_COLOR][pwg_pq] > 0)
+	  pwg_om = _PWG_OUTPUT_MODE_COLOR;
+	else
+	{
+	  pwg_pq = _PWG_PRINT_QUALITY_NORMAL;
+	  pwg_om = _PWG_OUTPUT_MODE_COLOR;
+	}
+      }
+
+      if (pwg->num_presets[pwg_om][pwg_pq] > 0)
+      {
+       /*
+	* Copy the preset options as long as the corresponding names are not
+	* already defined in the IPP request...
+	*/
+
+	for (i = pwg->num_presets[pwg_om][pwg_pq],
+		 preset = pwg->presets[pwg_om][pwg_pq];
+	     i > 0;
+	     i --, preset ++)
+	{
+	  if (!cupsGetOption(preset->name, num_options, options))
+	    ppd_mark_option(ppd, preset->name, preset->value);
+	}
+      }
+    }
+
+    if (output_bin && !cupsGetOption("OutputBin", num_options, options) &&
+	(ppd_keyword = _pwgGetOutputBin(pwg, output_bin)) != NULL) 
+    {
+     /*
+      * Map output-bin to OutputBin...
+      */
+
+      ppd_mark_option(ppd, "OutputBin", ppd_keyword);
+    }
+
+    if (sides && pwg->sides_option &&
+        !cupsGetOption(pwg->sides_option, num_options, options))
+    {
+     /*
+      * Map sides to duplex option...
+      */
+
+      if (!strcmp(sides, "one-sided"))
+        ppd_mark_option(ppd, pwg->sides_option, pwg->sides_1sided);
+      else if (!strcmp(sides, "two-sided-long-edge"))
+        ppd_mark_option(ppd, pwg->sides_option, pwg->sides_2sided_long);
+      else if (!strcmp(sides, "two-sided-short-edge"))
+        ppd_mark_option(ppd, pwg->sides_option, pwg->sides_2sided_short);
     }
   }
 
@@ -241,98 +272,11 @@ cupsMarkOptions(
 
   for (i = num_options, optptr = options; i > 0; i --, optptr ++)
     if (!strcasecmp(optptr->name, "media") ||
-        !strcasecmp(optptr->name, "media-col"))
+        !strcasecmp(optptr->name, "output-bin") ||
+	!strcasecmp(optptr->name, "output-mode") ||
+	!strcasecmp(optptr->name, "print-quality") ||
+	!strcasecmp(optptr->name, "sides"))
       continue;
-    else if (!strcasecmp(optptr->name, "sides"))
-    {
-      for (j = 0;
-           j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0]));
-	   j ++)
-        if (cupsGetOption(duplex_options[j], num_options, options))
-	  break;
-
-      if (j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])))
-      {
-       /*
-        * Don't override the PPD option with the IPP attribute...
-	*/
-
-        continue;
-      }
-
-      if (!strcasecmp(optptr->value, "one-sided"))
-      {
-       /*
-        * Mark the appropriate duplex option for one-sided output...
-	*/
-
-        for (j = 0;
-	     j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0]));
-	     j ++)
-	  if ((option = ppdFindOption(ppd, duplex_options[j])) != NULL)
-	    break;
-
-	if (j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])))
-	{
-          for (k = 0;
-	       k < (int)(sizeof(duplex_one) / sizeof(duplex_one[0]));
-	       k ++)
-            if (ppdFindChoice(option, duplex_one[k]))
-	    {
-	      ppd_mark_option(ppd, duplex_options[j], duplex_one[k]);
-	      break;
-            }
-        }
-      }
-      else if (!strcasecmp(optptr->value, "two-sided-long-edge"))
-      {
-       /*
-        * Mark the appropriate duplex option for two-sided-long-edge output...
-	*/
-
-        for (j = 0;
-	     j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0]));
-	     j ++)
-	  if ((option = ppdFindOption(ppd, duplex_options[j])) != NULL)
-	    break;
-
-	if (j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])))
-	{
-          for (k = 0;
-	       k < (int)(sizeof(duplex_two_long) / sizeof(duplex_two_long[0]));
-	       k ++)
-            if (ppdFindChoice(option, duplex_two_long[k]))
-	    {
-	      ppd_mark_option(ppd, duplex_options[j], duplex_two_long[k]);
-	      break;
-            }
-        }
-      }
-      else if (!strcasecmp(optptr->value, "two-sided-short-edge"))
-      {
-       /*
-        * Mark the appropriate duplex option for two-sided-short-edge output...
-	*/
-
-        for (j = 0;
-	     j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0]));
-	     j ++)
-	  if ((option = ppdFindOption(ppd, duplex_options[j])) != NULL)
-	    break;
-
-	if (j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])))
-	{
-          for (k = 0;
-	       k < (int)(sizeof(duplex_two_short) / sizeof(duplex_two_short[0]));
-	       k ++)
-            if (ppdFindChoice(option, duplex_two_short[k]))
-	    {
-	      ppd_mark_option(ppd, duplex_options[j], duplex_two_short[k]);
-	      break;
-            }
-        }
-      }
-    }
     else if (!strcasecmp(optptr->name, "resolution") ||
              !strcasecmp(optptr->name, "printer-resolution"))
     {
@@ -344,11 +288,6 @@ cupsMarkOptions(
       ppd_mark_option(ppd, "CNRes_PGP", optptr->value);
       	/* Canon */
     }
-    else if (!strcasecmp(optptr->name, "output-bin"))
-    {
-      if (!cupsGetOption("OutputBin", num_options, options))
-        ppd_mark_option(ppd, "OutputBin", optptr->value);
-    }
     else if (!strcasecmp(optptr->name, "multiple-document-handling"))
     {
       if (!cupsGetOption("Collate", num_options, options) &&
@@ -421,7 +360,7 @@ cupsMarkOptions(
     else
       ppd_mark_option(ppd, optptr->name, optptr->value);
 
-  debug_marked(ppd, "After...");
+  ppd_debug_marked(ppd, "After...");
 
   return (ppdConflicts(ppd) > 0);
 }
@@ -581,7 +520,10 @@ ppdMarkDefaults(ppd_file_t *ppd)	/* I - PPD file record */
   for (c = (ppd_choice_t *)cupsArrayFirst(ppd->marked);
        c;
        c = (ppd_choice_t *)cupsArrayNext(ppd->marked))
+  {
     cupsArrayRemove(ppd->marked, c);
+    c->marked = 0;
+  }
 
  /*
   * Then repopulate it with the defaults...
@@ -668,6 +610,7 @@ ppdNextOption(ppd_file_t *ppd)		/* I - PPD file */
  * This function looks for strings of the form:
  *
  *     *option choice ... *optionN choiceN
+ *     property value ... propertyN valueN
  *
  * It stops when it finds a string that doesn't match this format.
  */
@@ -676,10 +619,11 @@ int					/* O  - Number of options */
 _ppdParseOptions(
     const char    *s,			/* I  - String to parse */
     int           num_options,		/* I  - Number of options */
-    cups_option_t **options)		/* IO - Options */
+    cups_option_t **options,		/* IO - Options */
+    _ppd_parse_t  which)		/* I  - What to parse */
 {
-  char	option[PPD_MAX_NAME],		/* Current option */
-	choice[PPD_MAX_NAME],		/* Current choice */
+  char	option[PPD_MAX_NAME * 2 + 1],	/* Current option/property */
+	choice[PPD_MAX_NAME],		/* Current choice/value */
 	*ptr;				/* Pointer into option or choice */
 
 
@@ -687,8 +631,8 @@ _ppdParseOptions(
     return (num_options);
 
  /*
-  * Read all of the "*Option Choice" pairs from the string, marking PPD
-  * options as we go...
+  * Read all of the "*Option Choice" and "property value" pairs from the
+  * string, add them to an options array as we go...
   */
 
   while (*s)
@@ -697,22 +641,18 @@ _ppdParseOptions(
     * Skip leading whitespace...
     */
 
-    while (isspace(*s & 255))
+    while (_cups_isspace(*s))
       s ++;
 
-    if (*s != '*')
-      break;
-
    /*
-    * Get the option name...
+    * Get the option/property name...
     */
 
-    s ++;
     ptr = option;
-    while (*s && !isspace(*s & 255) && ptr < (option + sizeof(option) - 1))
+    while (*s && !_cups_isspace(*s) && ptr < (option + sizeof(option) - 1))
       *ptr++ = *s++;
 
-    if (ptr == s)
+    if (ptr == s || !_cups_isspace(*s))
       break;
 
     *ptr = '\0';
@@ -721,23 +661,29 @@ _ppdParseOptions(
     * Get the choice...
     */
 
-    while (isspace(*s & 255))
+    while (_cups_isspace(*s))
       s ++;
 
     if (!*s)
       break;
 
     ptr = choice;
-    while (*s && !isspace(*s & 255) && ptr < (choice + sizeof(choice) - 1))
+    while (*s && !_cups_isspace(*s) && ptr < (choice + sizeof(choice) - 1))
       *ptr++ = *s++;
 
+    if (*s && !_cups_isspace(*s))
+      break;
+
     *ptr = '\0';
 
    /*
     * Add it to the options array...
     */
 
-    num_options = cupsAddOption(option, choice, num_options, options);
+    if (option[0] == '*' && which != _PPD_PARSE_PROPERTIES)
+      num_options = cupsAddOption(option + 1, choice, num_options, options);
+    else if (option[0] != '*' && which != _PPD_PARSE_OPTIONS)
+      num_options = cupsAddOption(option, choice, num_options, options);
   }
 
   return (num_options);
@@ -746,11 +692,11 @@ _ppdParseOptions(
 
 #ifdef DEBUG
 /*
- * 'debug_marked()' - Output the marked array to stdout...
+ * 'ppd_debug_marked()' - Output the marked array to stdout...
  */
 
 static void
-debug_marked(ppd_file_t *ppd,		/* I - PPD file data */
+ppd_debug_marked(ppd_file_t *ppd,		/* I - PPD file data */
              const char *title)		/* I - Title for list */
 {
   ppd_choice_t	*c;			/* Current choice */
@@ -806,7 +752,7 @@ ppd_mark_choices(ppd_file_t *ppd,	/* I - PPD file */
     return;
 
   options     = NULL;
-  num_options = _ppdParseOptions(s, 0, &options);
+  num_options = _ppdParseOptions(s, 0, &options, 0);
 
   for (i = num_options, option = options; i > 0; i --, option ++)
     ppd_mark_option(ppd, option->name, option->value);
@@ -1149,106 +1095,5 @@ ppd_mark_option(ppd_file_t *ppd,	/* I - PPD file */
 
 
 /*
- * 'ppd_mark_size()' - Quickly mark a page size without checking for conflicts.
- *
- * This function is also responsible for mapping PWG/ISO/IPP size names to the
- * PPD file...
- */
-
-static void
-ppd_mark_size(ppd_file_t *ppd,		/* I - PPD file */
-              const char *size)		/* I - Size name */
-{
-  int			i;		/* Looping var */
-  _cups_pwg_media_t	*pwgmedia;	/* PWG media information */
-  ppd_size_t		*ppdsize;	/* Current PPD size */
-  double		dw, dl;		/* Difference in width and height */
-  double		width,		/* Width to find */
-			length;		/* Length to find */
-  char			width_str[256],	/* Width in size name */
-			length_str[256],/* Length in size name */
-			units[256],	/* Units in size name */
-			custom[256];	/* Custom size */
-  struct lconv		*loc;		/* Localization data */
-
-
- /*
-  * See if this is a PPD size...
-  */
-
-  if (!strncasecmp(size, "Custom.", 7) || ppdPageSize(ppd, size))
-  {
-    ppd_mark_option(ppd, "PageSize", size);
-    return;
-  }
-
- /*
-  * Nope, try looking up the PWG or legacy (IPP/ISO) size name...
-  */
-
-  if ((pwgmedia = _cupsPWGMediaByName(size)) == NULL)
-    pwgmedia = _cupsPWGMediaByLegacy(size);
-
-  if (pwgmedia)
-  {
-    width  = pwgmedia->width;
-    length = pwgmedia->length;
-  }
-  else if (sscanf(size, "%*[^_]_%*[^_]_%255[0-9.]x%255[0-9.]%s", width_str,
-                  length_str, units) == 3)
-  {
-   /*
-    * Got a "self-describing" name that isn't in our table...
-    */
-
-    loc    = localeconv();
-    width  = _cupsStrScand(width_str, NULL, loc);
-    length = _cupsStrScand(length_str, NULL, loc);
-
-    if (!strcmp(units, "in"))
-    {
-      width  *= 72.0;
-      length *= 72.0;
-    }
-    else if (!strcmp(units, "mm"))
-    {
-      width  *= 25.4 / 72.0;
-      length *= 25.4 / 72.0;
-    }
-    else
-      return;
-  }
-  else
-    return;
-
- /*
-  * Search the PPD file for a matching size...
-  */
-
-  for (i = ppd->num_sizes, ppdsize = ppd->sizes; i > 0; i --, ppdsize ++)
-  {
-    dw = ppdsize->width - width;
-    dl = ppdsize->length - length;
-
-    if (dw > -5.0 && dw < 5.0 && dl > -5.0 && dl < 5.0)
-    {
-      ppd_mark_option(ppd, "PageSize", ppdsize->name);
-      return;
-    }
-  }
-
- /*
-  * No match found; if custom sizes are supported, set a custom size...
-  */
-
-  if (ppd->variable_sizes)
-  {
-    snprintf(custom, sizeof(custom), "Custom.%dx%d", (int)width, (int)length);
-    ppd_mark_option(ppd, "PageSize", custom);
-  }
-}
-
-
-/*
- * End of "$Id: mark.c 8912 2009-12-08 02:13:42Z mike $".
+ * End of "$Id: mark.c 9426 2011-01-06 22:48:42Z mike $".
  */
