@@ -1,9 +1,9 @@
 /*
- * "$Id: testppd.c 8672 2009-05-22 17:10:00Z mike $"
+ * "$Id: testppd.c 9586 2011-03-04 21:43:50Z mike $"
  *
- *   PPD test program for the Common UNIX Printing System (CUPS).
+ *   PPD test program for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -23,13 +23,12 @@
  * Include necessary headers...
  */
 
+#include <cups/cups.h>
+#include <cups/string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <cups/string.h>
 #include <sys/stat.h>
 #include <errno.h>
-#include "cups.h"
-#include "pwgmedia.h"
 #ifdef WIN32
 #  include <io.h>
 #else
@@ -58,6 +57,11 @@ static const char	*default_code =
 			"%%EndFeature\n"
 			"} stopped cleartomark\n"
 			"[{\n"
+			"%%BeginFeature: *MediaType Plain\n"
+			"MediaType=Plain\n"
+			"%%EndFeature\n"
+			"} stopped cleartomark\n"
+			"[{\n"
 			"%%BeginFeature: *IntOption None\n"
 			"%%EndFeature\n"
 			"} stopped cleartomark\n"
@@ -77,6 +81,11 @@ static const char	*custom_code =
 			"%%EndFeature\n"
 			"} stopped cleartomark\n"
 			"[{\n"
+			"%%BeginFeature: *MediaType Plain\n"
+			"MediaType=Plain\n"
+			"%%EndFeature\n"
+			"} stopped cleartomark\n"
+			"[{\n"
 			"%%BeginFeature: *IntOption None\n"
 			"%%EndFeature\n"
 			"} stopped cleartomark\n"
@@ -145,7 +154,6 @@ main(int  argc,				/* I - Number of command-line arguments */
 		maxsize,		/* Maximum size */
 		*size;			/* Current size */
   ppd_attr_t	*attr;			/* Current attribute */
-  _cups_pwg_media_t *pwgmedia;		/* PWG media size */
 
 
   status = 0;
@@ -437,15 +445,30 @@ main(int  argc,				/* I - Number of command-line arguments */
     else
       puts("PASS");
 
-    fputs("_cupsPWGMediaBySize(842, 1191): ", stdout);
-    if ((pwgmedia = _cupsPWGMediaBySize(842, 1191)) == NULL)
+    fputs("cupsMarkOptions(media=oe_letter-fullbleed_8.5x11in): ", stdout);
+    num_options = cupsAddOption("media", "oe_letter-fullbleed_8.5x11in", 0,
+                                &options);
+    cupsMarkOptions(ppd, num_options, options);
+    cupsFreeOptions(num_options, options);
+
+    size = ppdPageSize(ppd, NULL);
+    if (!size || strcmp(size->name, "Letter.Fullbleed"))
     {
-      puts("FAIL (not found)");
+      printf("FAIL (%s)\n", size ? size->name : "unknown");
       status ++;
     }
-    else if (strcmp(pwgmedia->pwg, "iso_a3_297x420mm"))
+    else
+      puts("PASS");
+
+    fputs("cupsMarkOptions(media=A4): ", stdout);
+    num_options = cupsAddOption("media", "A4", 0, &options);
+    cupsMarkOptions(ppd, num_options, options);
+    cupsFreeOptions(num_options, options);
+
+    size = ppdPageSize(ppd, NULL);
+    if (!size || strcmp(size->name, "A4"))
     {
-      printf("FAIL (%s)\n", pwgmedia->pwg);
+      printf("FAIL (%s)\n", size ? size->name : "unknown");
       status ++;
     }
     else
@@ -706,7 +729,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     }
     else
       puts("FAIL (No conflicts!)");
-    
+
     fputs("ppdInstallableConflict(): ", stdout);
     if (ppdInstallableConflict(ppd, "Duplex", "DuplexNoTumble") &&
         !ppdInstallableConflict(ppd, "Duplex", "None"))
@@ -822,10 +845,23 @@ main(int  argc,				/* I - Number of command-line arguments */
 
     if (!strncmp(argv[1], "-d", 2))
     {
-      filename = cupsGetPPD(argv[1] + 2);
+      const char *printer;		/* Printer name */
+
+      if (argv[1][2])
+	printer = argv[1] + 2;
+      else if (argv[2])
+	printer = argv[2];
+      else
+      {
+        puts("Usage: ./testppd -d printer");
+	return (1);
+      }
+
+      filename = cupsGetPPD(printer);
+
       if (!filename)
       {
-        printf("%s: %s\n", argv[1], cupsLastErrorString());
+        printf("%s: %s\n", printer, cupsLastErrorString());
         return (1);
       }
     }
@@ -1041,5 +1077,5 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 
 /*
- * End of "$Id: testppd.c 8672 2009-05-22 17:10:00Z mike $".
+ * End of "$Id: testppd.c 9586 2011-03-04 21:43:50Z mike $".
  */
