@@ -1,5 +1,5 @@
 /*
- * "$Id: image-gif.c 7421 2008-04-01 20:32:12Z mike $"
+ * "$Id: image-gif.c 9841 2011-06-20 20:37:51Z mike $"
  *
  *   GIF image routines for the Common UNIX Printing System (CUPS).
  *
@@ -353,7 +353,7 @@ gif_get_code(FILE *fp,			/* I - File to read from */
     * Read in another buffer...
     */
 
-    if ((count = gif_get_block (fp, buf + last_byte)) <= 0)
+    if ((count = gif_get_block(fp, buf + last_byte)) <= 0)
     {
      /*
       * Whoops, no more data!
@@ -372,7 +372,6 @@ gif_get_code(FILE *fp,			/* I - File to read from */
     lastbit   = last_byte * 8;
   }
 
-  ret = 0;
   for (ret = 0, i = curbit + code_size - 1, j = code_size;
        j > 0;
        i --, j --)
@@ -583,19 +582,13 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
     gif_get_code(fp, 0, 1);
 
    /*
-    * Wipe the decompressor table...
+    * Wipe the decompressor table (already mostly 0 due to the calloc above...)
     */
 
     fresh = 1;
 
-    for (i = 0; i < clear_code; i ++)
-    {
-      table[0][i] = 0;
+    for (i = 1; i < clear_code; i ++)
       table[1][i] = i;
-    }
-
-    for (; i < 4096; i ++)
-      table[0][i] = table[1][0] = 0;
 
     sp = stack;
 
@@ -606,27 +599,30 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
     fresh = 0;
 
     do
+    {
       firstcode = oldcode = gif_get_code(fp, code_size, 0);
+    }
     while (firstcode == clear_code);
 
-    return (firstcode);
+    return (firstcode & 255);
   }
+  else if (!table)
+    return (0);
 
   if (sp > stack)
-    return (*--sp);
+    return ((*--sp) & 255);
 
-  while ((code = gif_get_code (fp, code_size, 0)) >= 0)
+  while ((code = gif_get_code(fp, code_size, 0)) >= 0)
   {
     if (code == clear_code)
     {
-      for (i = 0; i < clear_code; i ++)
-      {
-	table[0][i] = 0;
-	table[1][i] = i;
-      }
+     /*
+      * Clear/reset the compression table...
+      */
 
-      for (; i < 4096; i ++)
-	table[0][i] = table[1][i] = 0;
+      memset(table, 0, 2 * sizeof(gif_table_t));
+      for (i = 1; i < clear_code; i ++)
+	table[1][i] = i;
 
       code_size     = set_code_size + 1;
       max_code_size = 2 * clear_code;
@@ -636,12 +632,11 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
 
       firstcode = oldcode = gif_get_code(fp, code_size, 0);
 
-      return (firstcode);
+      return (firstcode & 255);
     }
-    else if (code == end_code)
+    else if (code == end_code || code > max_code)
     {
-      unsigned char	buf[260];
-
+      unsigned char	buf[260];	/* Block buffer */
 
       if (!gif_eof)
         while (gif_get_block(fp, buf) > 0);
@@ -651,7 +646,7 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
 
     incode = code;
 
-    if (code >= max_code)
+    if (code == max_code)
     {
       *sp++ = firstcode;
       code  = oldcode;
@@ -685,13 +680,13 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
     oldcode = incode;
 
     if (sp > stack)
-      return (*--sp);
+      return ((*--sp) & 255);
   }
 
-  return (code);
+  return (code & 255);
 }
 
 
 /*
- * End of "$Id: image-gif.c 7421 2008-04-01 20:32:12Z mike $".
+ * End of "$Id: image-gif.c 9841 2011-06-20 20:37:51Z mike $".
  */
