@@ -1,9 +1,9 @@
 /*
- * "$Id: image-zoom.c 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: image-zoom.c 9503 2011-01-22 00:07:22Z mike $"
  *
- *   cupsImage zoom routines for the Common UNIX Printing System (CUPS).
+ *   cupsImage zoom routines for CUPS.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1993-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -136,7 +136,7 @@ _cupsImageZoomNew(
     z->ystep   = z->height / z->ysize;
     z->yincr   = 1;
     z->instep  = z->xstep * z->depth;
-    z->inincr  = z->xincr * z->depth;
+    z->inincr  = /* z->xincr * */ z->depth; /* z->xincr is always 1 */
 
     if (z->width < img->ysize)
       z->xmax = z->width;
@@ -163,7 +163,7 @@ _cupsImageZoomNew(
     z->ystep   = z->height / z->ysize;
     z->yincr   = 1;
     z->instep  = z->xstep * z->depth;
-    z->inincr  = z->xincr * z->depth;
+    z->inincr  = /* z->xincr * */ z->depth; /* z->xincr is always 1 */
 
     if (z->width < img->xsize)
       z->xmax = z->width;
@@ -357,5 +357,5 @@ zoom_nearest(cups_izoom_t *z,		/* I - Zoom record to fill */
 
 
 /*
- * End of "$Id: image-zoom.c 7721 2008-07-11 22:48:49Z mike $".
+ * End of "$Id: image-zoom.c 9503 2011-01-22 00:07:22Z mike $".
  */
