@@ -1,9 +1,9 @@
 /*
- * "$Id: lpadmin.c 9258 2010-08-13 01:34:04Z mike $"
+ * "$Id: lpadmin.c 9479 2011-01-13 00:01:48Z mike $"
  *
  *   "lpadmin" command for CUPS.
  *
- *   Copyright 2007-2010 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -1240,7 +1240,7 @@ set_printer_options(
     ppdMarkDefaults(ppd);
     cupsMarkOptions(ppd, num_options, options);
 
-    if ((out = cupsTempFile2(tempfile, sizeof(tempfile))) < 0)
+    if ((out = cupsTempFile2(tempfile, sizeof(tempfile))) == NULL)
     {
       _cupsLangPrintError(_("ERROR: Unable to create temporary file"));
       ippDelete(request);
@@ -1390,5 +1390,5 @@ validate_name(const char *name)		/* I - Name to check */
 
 
 /*
- * End of "$Id: lpadmin.c 9258 2010-08-13 01:34:04Z mike $".
+ * End of "$Id: lpadmin.c 9479 2011-01-13 00:01:48Z mike $".
  */
